package org.jeecg.modules.system.app.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.system.app.dto.AppCoursesDTO;
import org.jeecg.modules.system.app.entity.AppCourses;
import org.jeecg.modules.system.app.entity.AppCoursesPriceRules;
import org.jeecg.modules.system.app.mapper.AppCoursesMapper;
import org.jeecg.modules.system.app.mapper.AppCoursesPriceRulesMapper;
import org.jeecg.modules.system.app.service.IAppCoureseService;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.annotation.Resource;
import java.util.List;

import static org.jeecg.common.constant.CommonConstant.SC_INTERNAL_SERVER_ERROR_500;

/**
 * @Description: 课程/培训表
 * @Author: jeecg-boot
 * @Date: 2025-07-03
 * @Version: V1.0
 */
@Service
public class AppCoureseServiceImpl extends ServiceImpl<AppCoursesMapper, AppCourses> implements IAppCoureseService {
    @Resource
    AppCoursesPriceRulesMapper coursesPriceRulesMapper;

    @Override
    public Boolean saveWitchPriceRules(AppCoursesDTO appCoursesDTO) {
        AppCourses course = appCoursesDTO.getCourse();
        if (null == course.getCategoryId()) throw new JeecgBootException("请选择课程类别");
        LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        course.setOrgCode(loginUser.getOrgCode());
        course.setTenantId(loginUser.getOrgId());
        int saveCourseResult = baseMapper.insert(course);
        if (saveCourseResult < 1) {
            throw new JeecgBootException("课程信息保存失败");
        }
        List<AppCoursesPriceRules> priceRulesList = appCoursesDTO.getPriceRulesList();
        for (AppCoursesPriceRules rule : priceRulesList) {
            rule.setCoursesId(course.getId());
            rule.setOrgCode(course.getOrgCode());
            rule.setTenantId(course.getTenantId());
            int savePriceResult = coursesPriceRulesMapper.insert(rule); // 保证每个操作都在事务中
            if (savePriceResult < 1) {
                throw new JeecgBootException("课程价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean editWitchPriceRules(AppCoursesDTO appCoursesDTO) {
        AppCourses course = appCoursesDTO.getCourse();
        if (null == course.getCategoryId()) throw new JeecgBootException("请选择课程类别");
        LoginUser loginUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        AppCourses dbCourse = baseMapper.selectById(course.getId());
        checkPermission(loginUser, dbCourse);
        course.setOrgCode(loginUser.getOrgCode());
        course.setTenantId(loginUser.getOrgId());
        int updateCourseResult = baseMapper.updateById(course);
        if (updateCourseResult < 1) {
            throw new JeecgBootException("课程信息保存失败");
        }
        List<AppCoursesPriceRules> priceRulesList = appCoursesDTO.getPriceRulesList();
        for (AppCoursesPriceRules rule : priceRulesList) {
            rule.setCoursesId(course.getId());
            rule.setOrgCode(course.getOrgCode());
            rule.setTenantId(course.getTenantId());
            int savePriceResult = coursesPriceRulesMapper.updateById(rule);
            if (savePriceResult < 1) {
                throw new JeecgBootException("课程价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public AppCoursesDTO queryWitchPriceRulesById(String id) {
        AppCourses dbCourse = baseMapper.selectById(id);
        if (null==dbCourse) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        checkPermission(sysUser,dbCourse);
        List<AppCoursesPriceRules> priceRulesList = coursesPriceRulesMapper.selectList(Wrappers.<AppCoursesPriceRules>lambdaQuery().eq(AppCoursesPriceRules::getCoursesId, id));
        return new AppCoursesDTO(dbCourse, priceRulesList);
    }

    @Override
    public Boolean deleteWitchPriceRulesById(String id) {
        AppCourses dbCourse = baseMapper.selectById(id);
        if (null==dbCourse) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        checkPermission(sysUser,dbCourse);
        int deleteCourseResult = baseMapper.deleteById(id);
        if (deleteCourseResult < 1) {
            throw new JeecgBootException("课程信息删除失败", SC_INTERNAL_SERVER_ERROR_500);
        }
        int deletePriceResult = coursesPriceRulesMapper.delete(Wrappers.<AppCoursesPriceRules>lambdaQuery().eq(AppCoursesPriceRules::getCoursesId, id));
        if (deletePriceResult<1) throw new JeecgBootException("课程价格规则删除失败", SC_INTERNAL_SERVER_ERROR_500);
        return Boolean.TRUE;
    }

    /**
     *  校验权限
     * @param loginUser
     * @param course
     */
    private void checkPermission(LoginUser loginUser, AppCourses course) {
        if (!loginUser.getOrgCode().equals(course.getOrgCode())) throw new JeecgBootException("无权限修改", SC_INTERNAL_SERVER_ERROR_500);
    }
}
